/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web; // NOPMD

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankDetailsPage;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankLoginPage;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankTransactionsPage;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractSeleniumWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.LoginPage;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Stellt das Webseiten-Interface dar.
 *
 * @author Frank Mnster
 */
public final class TargoBankWebSite extends AbstractSeleniumWebSite<TargoBankWebAccountInformation, TargoBankMessages> {

    /**
     * Unsere BceeWebSite instance
     */
    private static TargoBankWebSite webSite;

    /**
     * Das Konto
     */
    private final Konto konto;

    /**
     * Das Login/die Kundenkennung
     */
    private final String kundenkennung;

    /**
     * Liefert die Instance der WebSeite
     *
     * @param konto
     * @return die WebSeite
     * @throws WebSiteException
     * @throws ApplicationException
     */
    public static TargoBankWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

        if (TargoBankWebSite.webSite == null) { // NOPMD
            TargoBankWebSite.webSite = new TargoBankWebSite(konto);
        } else {
            // Ist das Konto dasselbe
            try {
                if (TargoBankWebSite.webSite.konto.equals(konto)) {
                    // Aktualisiere Startdatum der Umsatzabfrage
                    final Date lastTransacDate = Utils.getLastUmsatzDate(konto,
                            TargoBankWebSite.webSite.getStartdate());
                    TargoBankWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
                } else {
                    TargoBankWebSite.webSite.logoff();
                    TargoBankWebSite.webSite = new TargoBankWebSite(konto);
                }
            } catch (final RemoteException e) {
                final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
                Logger.error(msg, e);
                throw new ApplicationException(msg, e);
            }
        }

        return TargoBankWebSite.webSite;
    }

    /**
     * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der
     * TargoBank.
     *
     * @param konto
     * @param login
     * @param cardCompany
     * @param cardAccount
     * @param cardNumber
     * @throws WebSiteException
     * @throws ApplicationException
     */
    private TargoBankWebSite(final Konto konto) throws WebSiteException, ApplicationException {

        super(konto);
        this.konto = konto;

        String cardAccount;
        String cardNumber;
        String cardCompany;
        String loginName;
        boolean stayLoggedIn;
        try {
            loginName = konto.getKundennummer();
            if (StringUtils.isEmpty(loginName)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.loginMissing")); //$NON-NLS-1$
            }
            cardAccount = konto.getMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, null);
            if (StringUtils.isEmpty(cardAccount)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardAccountMissing")); //$NON-NLS-1$
            }
            cardNumber = konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, null);
            if (StringUtils.isEmpty(cardNumber)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardNumberMissing")); //$NON-NLS-1$
            }
            cardNumber = cardNumber.replaceAll("\\s+", StringUtils.EMPTY); //$NON-NLS-1$
            cardCompany = konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, null);
            if (StringUtils.isEmpty(cardCompany)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardCompanyMissing")); //$NON-NLS-1$
            }
            stayLoggedIn = Utils
                    .getBooleanFromYesNoString(konto.getMeta(MahoeSynchronizeBackend.STAY_LOGGED_IN, Utils.TEXT_NEIN));

        } catch (final RemoteException e) {
            final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }

        this.kundenkennung = loginName;
        this.accountInfo.setCardCompany(cardCompany);
        this.accountInfo.setCardNumber(cardNumber);
        this.accountInfo.setCardAccount(cardAccount);
        this.accountInfo.setStayLoggedIn(stayLoggedIn);

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        Logger.info(I18N.tr("AbstractWebSite.transferNotSupported")); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public TargoBankWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        final TargoBankDetailsPage detailsPage = new TargoBankDetailsPage(this.webClient, this.accountInfo);

        this.accountInfo.setSaldo(detailsPage.getSaldo());
        this.accountInfo.setVerfuegungsrahmen(detailsPage.getCreditLimit());
        this.accountInfo.setSaldoAvailable(detailsPage.getAvailableAmount());
        this.accountInfo.setNextPayDay(detailsPage.getNextSubtraction());

        return this.accountInfo;
    }

    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        // final TargoBankAccountStatementFinder accountSheets = new
        // TargoBankAccountStatementFinder(this.webClient,
        // accountInfo);
        // accountSheets.findStatements();
        //
        // return accountSheets.getSheets();

        return new ArrayList<>();

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        // // open the account statements page
        // BankAccountStatementPage page = new
        // TargoBankAccountStatementsPage(this.webClient,
        // this.accountInfo.getCardAccount());
        //
        // // two month back only to be save
        // final Date from = new
        // Date(this.accountInfo.getLastTransactionDate().getTime() - 60 * 24 *
        // 60 * 60 * 1000);
        //
        // // get transactions from actual page
        // final List<Umsatz> transactions = page.getTransactions(from);
        //
        // // check if we need to load from other pages as well
        // while (page.accountLinksHasNextPage()) {
        // Logger.debug("More pages available"); //$NON-NLS-1$
        // page = page.getNextPage();
        // transactions.addAll(page.getTransactions(from));
        // }

        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        // final TargoBankWebSiteTransactionsImpl transImpl = new
        // TargoBankWebSiteTransactionsImpl(this.webClient,
        // this.accountInfo);

        // get a new TransactionsPage
        final TargoBankTransactionsPage transactionsPage = new TargoBankTransactionsPage(this.webClient,
                this.accountInfo);

        // return the found pre-book transactions
        return transactionsPage.getTransactions(this.accountInfo.getLastTransactionDate());
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        return new TargoBankLoginPage(this.webClient).isLoggedIn();

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], String...)
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        // first check if driver available
        // TODO: check if necessary
        this.checkSeleniumDriverSet(this.webClient);

        if (loginParams.length != 0) {
            throw new WebSiteException(I18N.tr("TargoBankWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }

        try {
            final LoginPage loginPage = new TargoBankLoginPage(this.webClient);
            loginPage.login(this.kundenkennung, password);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)
                    || e.getType().equals(WebSiteExceptionTypes.LOGIN_ERROR)) {
                // Passwort zurcksetzen
                PasswdCache.revoke(TargoBank.NAME, this.kundenkennung);
            }
            Arrays.fill(password, ' ');
            throw e;
        }
        Arrays.fill(password, ' ');

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        if (!isStillLoggedIn()) {
            return;
        }

        // Stay logged in for debug purpose
        if (!this.accountInfo.isStayLoggedIn()) {
            // call log off URL
            this.webClient.get(TargoBank.URL_LOGOFF);

            // check if log off was successful
            final String title = this.webClient.getTitle();
            if (title.startsWith("Online-Banking Logout")) { //$NON-NLS-1$
                Logger.info(I18N.tr("TargoBankWebSite.logoffSuccessful")); //$NON-NLS-1$
            } else {
                throw new WebSiteException(I18N.tr("TargoBankWebSite.logoffFailed"), //$NON-NLS-1$
                        WebSiteExceptionTypes.LOGOFF_FAILED);
            }
        }

        // finally save the cookies
        saveCookies(TargoBank.COOKIE_FILE);
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setzte das startDatum auf Januar 2016
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.set(2016, 0, 1);
        return startCalendar;
    }

}
